# src/gui_components.py
import tkinter as tk
from tkinter import ttk, Toplevel, messagebox
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg, NavigationToolbar2Tk
from matplotlib.widgets import RectangleSelector, EllipseSelector, PolygonSelector
from matplotlib.patches import Rectangle, Ellipse, Polygon, Circle
from matplotlib.path import Path as MplPath
from matplotlib.colors import LogNorm, Normalize
import matplotlib.lines as mlines
import threading
import time
import os
import json

ROI_COLORS = ['#FF3333', '#33FF33', '#3388FF', '#FFFF33', '#FF33FF', '#33FFFF', '#FF8833']

class PlotManager:
    def __init__(self, parent_frame, app_instance):
        self.parent = parent_frame
        self.app = app_instance
        self.selector = None
        self.roi_list = [] 
        self.temp_roi = None
        
        try:
             from .plot_window import ROIPlotWindow
        except ImportError:
             try:
                 from plot_window import ROIPlotWindow
             except ImportError:
                 pass
        
        if 'ROIPlotWindow' in locals():
            self.plot_window_controller = ROIPlotWindow(self.app.root)
        else:
            self.plot_window_controller = None
        
        self.is_calculating = False
        self.current_shape_mode = "rect" 
        self.is_drawing_bg = False
        self.ax_ref = None
        self.btn_draw_ref = None

        self.fig = plt.Figure(figsize=(5, 5), dpi=100)
        self.fig.patch.set_facecolor('#FFFFFF')
        self.toolbar_container = ttk.Frame(self.parent, style="White.TFrame")
        self.toolbar_container.pack(side="top", fill="x", pady=(0, 0)) # 紧贴顶部


        self.canvas = FigureCanvasTkAgg(self.fig, master=self.parent)
        self.canvas_widget = self.canvas.get_tk_widget()
        self.canvas_widget.pack(side="top", fill="both", expand=True) # side="top" 确保在下方
        
        self.ax = self.fig.add_subplot(111)
        self.ax.axis('off')
        self.im_object = None
        self.cbar = None
        self.toolbar = None


    def apply_theme(self, bg_color, fg_color):
        """
        [新增] 动态切换 Matplotlib 的颜色主题
        """
        # 1. 设置 Figure 背景
        self.fig.patch.set_facecolor(bg_color)
        
        # 2. 设置 Axes 背景和边框
        self.ax.set_facecolor(bg_color)
        self.ax.spines['bottom'].set_color(fg_color)
        self.ax.spines['top'].set_color(fg_color) 
        self.ax.spines['left'].set_color(fg_color)
        self.ax.spines['right'].set_color(fg_color)
        
        # 3. 设置文字和刻度颜色
        self.ax.xaxis.label.set_color(fg_color)
        self.ax.yaxis.label.set_color(fg_color)
        self.ax.tick_params(axis='x', colors=fg_color)
        self.ax.tick_params(axis='y', colors=fg_color)
        self.ax.title.set_color(fg_color)
        
        # 4. 如果有 Colorbar，也要处理 (比较麻烦，通常重建比较好，或者只改 Label)
        if self.cbar:
            self.cbar.ax.yaxis.set_tick_params(color=fg_color, labelcolor=fg_color)
            self.cbar.ax.yaxis.label.set_color(fg_color)
        
        # 5. [修改] 更新 Toolbar 背景
        # 我们不使用传入的 plot_bg (太黑了)，而是去 app 里拿 toolbar_bg
        if self.toolbar:
            try:
                mode = self.app.current_theme
                c = self.app.THEME_COLORS[mode]
                tb_bg = c.get("toolbar_bg", "#F0F0F0")
                
                self.toolbar.config(background=tb_bg)
                # 坐标显示区域
                self.toolbar._message_label.config(background=tb_bg, foreground="black")
            except: pass

        self.canvas.draw_idle()



    def add_toolbar(self, parent_frame=None): # parent_frame 参数变为可选，兼容旧代码防止报错
        # 如果外部没传 parent (我们在 gui.py 里改成不传了)，就用内部创建的顶部容器
        target_frame = parent_frame if parent_frame else self.toolbar_container

        # 清理旧内容
        for child in target_frame.winfo_children():
            child.destroy()
            
        self.toolbar = NavigationToolbar2Tk(self.canvas, target_frame)
        
        # 样式配置 (保持不变)
        try:
            mode = self.app.current_theme
            c = self.app.THEME_COLORS[mode]
            bg_color = c.get("toolbar_bg", "#F0F0F0") 
            
            # 强制去掉 Matplotlib 工具栏自带的边框，使其更融合
            self.toolbar.config(background=bg_color, highlightthickness=0, bd=0)
            self.toolbar._message_label.config(background=bg_color, foreground="black")
        except:
            self.toolbar.config(background="#F0F0F0")
            
        self.toolbar.update()

    def show_logo(self, logo_path):
        self.fig.clear() 
        self.ax = self.fig.add_subplot(111)
        self.ax.axis('off')

        try:
            mode = self.app.current_theme
            c = self.app.THEME_COLORS[mode]
            self.fig.patch.set_facecolor(c["plot_bg"])
        except: pass


        self.im_object = None
        self.cbar = None
        if logo_path and os.path.exists(logo_path):
            try:
                img_arr = plt.imread(logo_path)
                self.ax.imshow(img_arr, alpha=0.15) 
            except Exception: pass
        self.canvas.draw()

    def init_image(self, shape, cmap="jet"):
        self.fig.clear()
        self.ax = self.fig.add_subplot(111)
        self.ax.axis('off')
        self.im_object = self.ax.imshow(np.zeros(shape), cmap=cmap)
        self.cbar = self.fig.colorbar(self.im_object, ax=self.ax, shrink=0.6, pad=0.02, label='Ratio Value')
        self.canvas.draw()

    def update_image(self, img_data, vmin, vmax, log_scale=False, title="", cbar_label=None):
        if self.im_object is None: return
        if log_scale:
            safe_vmin = max(vmin, 0.1)
            safe_vmax = max(vmax, safe_vmin * 1.1)
            norm = LogNorm(vmin=safe_vmin, vmax=safe_vmax)
        else:
            norm = Normalize(vmin=vmin, vmax=vmax)
            
        self.im_object.set_data(img_data)
        self.im_object.set_norm(norm)
        if self.cbar: 
            self.cbar.update_normal(self.im_object)
            if cbar_label: self.cbar.set_label(cbar_label)
        self.ax.set_title(title)
        self.canvas.draw_idle()

    def update_cmap(self, cmap_name, bg_color_str):
        if self.im_object is None: return
        cmap = plt.get_cmap(cmap_name).copy()
        bg = bg_color_str.lower()
        if bg in ["transparent", "trans"]: cmap.set_bad(alpha=0)
        else: cmap.set_bad(bg)
        self.im_object.set_cmap(cmap)
        self.canvas.draw_idle()

    def resize(self, event):
        self.canvas.resize(event)
    
    def get_ax(self): return self.ax


class RoiManager:
    def __init__(self, app_instance):
        self.app = app_instance
        self.selector = None
        self.roi_list = [] 
        self.temp_roi = None
        
        # 直线交互状态
        self.line_start_pt = None
        self.temp_line_artist = None
        self.dragging_roi = None       
        self.dragging_point_idx = -1   # 0=起点, 1=终点, 2=中点(平移)
        
        # 事件连接 ID
        self.cid_press = None
        self.cid_release = None
        self.cid_motion = None
        
        self.is_calculating = False
        self.current_shape_mode = "rect" 
        self.ax_ref = None
        self.btn_draw_ref = None
        self.is_drawing_bg = False
        self.last_drag_time = 0

    def set_draw_button(self, btn_widget):
        self.btn_draw_ref = btn_widget

    def connect(self, ax):
        self.ax_ref = ax
        self.roi_list = []
        self.temp_roi = None
        self._stop_selector()

    # =========================================================
    #  逻辑控制
    # =========================================================

    def set_mode(self, mode):
        """切换 ROI 模式"""
        if self.temp_roi: self._commit_temp_roi()
        self._stop_selector(clear_events=True)
        self.current_shape_mode = mode

        if mode == "line":
            self._connect_line_events()
            self.app.root.config(cursor="cross")
        elif mode:
            self.app.root.config(cursor="cross")
        else:
            self.app.root.config(cursor="")

    def cancel_drawing(self):
        """ESC 键按下时调用"""
        self.line_start_pt = None
        if self.temp_line_artist:
            self.temp_line_artist.remove()
            self.temp_line_artist = None
            
        self._stop_selector(clear_events=True)
        self.current_shape_mode = None
        
        if self.btn_draw_ref: self.btn_draw_ref.state(['!selected'])
        self.app.root.config(cursor="")
        self.app.plot_mgr.canvas.draw_idle()

    def _connect_line_events(self):
        if self.app.plot_mgr and self.cid_press is None:
            canvas = self.app.plot_mgr.canvas
            self.cid_press = canvas.mpl_connect('button_press_event', self._on_line_press)
            self.cid_release = canvas.mpl_connect('button_release_event', self._on_line_release)
            self.cid_motion = canvas.mpl_connect('motion_notify_event', self._on_line_drag)

    def _stop_selector(self, clear_events=False):
        if self.btn_draw_ref: self.btn_draw_ref.state(['!selected'])

        if self.selector:
            self.selector.set_active(False)
            self.selector.set_visible(False)
            self.selector = None
        
        if clear_events:
            canvas = self.app.plot_mgr.canvas
            if self.cid_press:   canvas.mpl_disconnect(self.cid_press); self.cid_press = None
            if self.cid_release: canvas.mpl_disconnect(self.cid_release); self.cid_release = None
            if self.cid_motion:  canvas.mpl_disconnect(self.cid_motion); self.cid_motion = None

        self.app.plot_mgr.canvas.draw_idle()

    def start_drawing(self, mode=None, is_background=False):
        if not self.ax_ref: return
        
        target_mode = mode if mode else self.current_shape_mode
        self.set_mode(target_mode)
        
        self.is_drawing_bg = is_background 
        if self.btn_draw_ref: self.btn_draw_ref.state(['selected']) 
        
        if self.current_shape_mode == "line":
            return

        if self.is_drawing_bg:
            props = dict(facecolor='black', edgecolor='gray', alpha=0.3, linestyle=':', linewidth=1, fill=True)
            line_props = dict(color='gray', linestyle=':', linewidth=1, alpha=0.8)
        else:
            next_id = len(self.roi_list) + 1
            color_idx = (next_id - 1) % len(ROI_COLORS)
            color = ROI_COLORS[color_idx]
            props = dict(facecolor=color, edgecolor='black', alpha=0.5, linestyle='--', linewidth=2, fill=True)
            line_props = dict(color='black', linestyle='--', linewidth=2, alpha=0.8)

        if self.current_shape_mode == "rect":
            self.selector = RectangleSelector(self.ax_ref, self._on_select_finalize, useblit=True, button=[1], minspanx=5, minspany=5, spancoords='pixels', interactive=True, props=props)
        elif self.current_shape_mode == "circle":
            self.selector = EllipseSelector(self.ax_ref, self._on_select_finalize, useblit=True, button=[1], minspanx=5, minspany=5, spancoords='pixels', interactive=True, props=props)
        elif self.current_shape_mode == "polygon":
            self.selector = PolygonSelector(self.ax_ref, self._on_poly_finalize, useblit=True, props=line_props)

        if self.selector:
            self.selector.set_active(True)

    # =========================================================
    #  直线交互逻辑 (支持平移与视觉增强)
    # =========================================================

    def _on_line_press(self, event):
        if event.inaxes != self.ax_ref: return
        if self.app.view_mode != "ratio" and self.app.view_mode != "ch1" and self.app.view_mode != "ch2" and not self.app.view_mode.startswith("aux"):
             return

        click_pt = (event.xdata, event.ydata)
        min_dist = float('inf')
        target_roi = None
        pt_idx = -1 
        
        threshold = 30.0 # 拾取阈值
        
        for roi in self.roi_list:
            if roi['type'] == 'line':
                p1, p2 = roi['params']
                
                # 计算端点距离
                d1 = np.hypot(p1[0]-click_pt[0], p1[1]-click_pt[1])
                d2 = np.hypot(p2[0]-click_pt[0], p2[1]-click_pt[1])
                
                # 计算中点距离 (平移手柄)
                mid_pt = ((p1[0]+p2[0])/2, (p1[1]+p2[1])/2)
                d_mid = np.hypot(mid_pt[0]-click_pt[0], mid_pt[1]-click_pt[1])
                
                # 优先检测端点，其次检测中点
                if d1 < threshold and d1 < min_dist:
                    min_dist = d1; target_roi = roi; pt_idx = 0
                if d2 < threshold and d2 < min_dist:
                    min_dist = d2; target_roi = roi; pt_idx = 1
                
                # 如果没点中端点，但点中了中间 (距离稍微放宽一点)
                if pt_idx == -1 and d_mid < threshold and d_mid < min_dist:
                    min_dist = d_mid; target_roi = roi; pt_idx = 2 # 2 代表中点
        
        if target_roi:
            self.dragging_roi = target_roi
            self.dragging_point_idx = pt_idx
            return 

        self.line_start_pt = (event.xdata, event.ydata)

    def _on_line_drag(self, event):
        if event.inaxes != self.ax_ref: return
        
        # A. 拖动模式
        if self.dragging_roi:
            p1, p2 = self.dragging_roi['params']
            new_pt = (event.xdata, event.ydata)
            
            # 0=起点, 1=终点, 2=平移
            if self.dragging_point_idx == 0:
                self.dragging_roi['params'] = (new_pt, p2)
            elif self.dragging_point_idx == 1:
                self.dragging_roi['params'] = (p1, new_pt)
            elif self.dragging_point_idx == 2:
                # 平移逻辑：计算中点位移量
                mid_old = ((p1[0]+p2[0])/2, (p1[1]+p2[1])/2)
                dx = new_pt[0] - mid_old[0]
                dy = new_pt[1] - mid_old[1]
                
                new_p1 = (p1[0] + dx, p1[1] + dy)
                new_p2 = (p2[0] + dx, p2[1] + dy)
                self.dragging_roi['params'] = (new_p1, new_p2)

            # 更新视觉 (Line + Circles)
            # patch_group 结构: [bg_line, fg_line, c_start, c_end, c_mid]
            group = self.dragging_roi['patch_group']
            bg_line, fg_line = group[0], group[1]
            c_start, c_end, c_mid = group[2], group[3], group[4] # 取出圆点
            
            curr_p1, curr_p2 = self.dragging_roi['params']
            curr_mid = ((curr_p1[0]+curr_p2[0])/2, (curr_p1[1]+curr_p2[1])/2)
            
            # 更新线
            bg_line.set_data([curr_p1[0], curr_p2[0]], [curr_p1[1], curr_p2[1]])
            fg_line.set_data([curr_p1[0], curr_p2[0]], [curr_p1[1], curr_p2[1]])
            
            # 更新圆点位置
            c_start.center = curr_p1
            c_end.center = curr_p2
            c_mid.center = curr_mid
            
            self.app.plot_mgr.canvas.draw_idle()
            
            if hasattr(self.app, 'update_kymograph_for_roi'):
                self.app.update_kymograph_for_roi(self.dragging_roi)
            return

        # B. 画新线预览
        if self.line_start_pt:
            x0, y0 = self.line_start_pt
            x1, y1 = event.xdata, event.ydata
            
            if self.temp_line_artist: self.temp_line_artist.remove()
            self.temp_line_artist = mlines.Line2D([x0, x1], [y0, y1], color='yellow', linestyle='--')
            self.ax_ref.add_line(self.temp_line_artist)
            self.app.plot_mgr.canvas.draw_idle()

    def _on_line_release(self, event):
        if self.dragging_roi:
            self.dragging_roi = None
            self.dragging_point_idx = -1
            return 

        if self.line_start_pt:
            if self.temp_line_artist:
                self.temp_line_artist.remove()
                self.temp_line_artist = None
            
            if event.xdata is not None and event.ydata is not None:
                end_pt = (event.xdata, event.ydata)
                dist = np.hypot(end_pt[0]-self.line_start_pt[0], end_pt[1]-self.line_start_pt[1])
                
                if dist > 5: 
                    params = (self.line_start_pt, end_pt)
                    next_id = len(self.roi_list) + 1
                    color = ROI_COLORS[(next_id - 1) % len(ROI_COLORS)]
                    patch_group = self._create_high_contrast_roi("line", params, color)

                    new_roi = {
                        'type': 'line', 
                        'patch_group': patch_group, 
                        'mask': None, 
                        'color': color, 
                        'id': next_id,
                        'params': params
                    }
                    self.roi_list.append(new_roi)
                    self.app.plot_mgr.canvas.draw_idle()
                    # 移除自动弹窗
            
            self.line_start_pt = None

    # =========================================================

    def _update_temp_roi_data(self, extents):
        xmin, xmax, ymin, ymax = extents
        if abs(xmax - xmin) < 1.0 or abs(ymax - ymin) < 1.0: return False
        
        if self.current_shape_mode == "rect": params = (xmin, ymin, xmax-xmin, ymax-ymin)
        elif self.current_shape_mode == "circle":
            w = xmax - xmin; h = ymax - ymin; cx = xmin + w/2; cy = ymin + h/2
            params = ((cx, cy), w, h)
        else: return False

        mask = self._generate_mask(self.current_shape_mode, params)
        if mask is None: return False
        next_id = len(self.roi_list) + 1
        color = ROI_COLORS[(next_id - 1) % len(ROI_COLORS)]
        self.temp_roi = {'type': self.current_shape_mode, 'params': params, 'mask': mask, 'color': color, 'id_display': next_id}
        return True

    def _on_drag_update(self, event):
        if not self.selector or not self.selector.active: return
        if not event.inaxes: return
        if not self.app.live_plot_var.get(): return
        now = time.time()
        if now - self.last_drag_time < 0.1: return
        if self.is_calculating: return
        self.last_drag_time = now
        try:
            if self._update_temp_roi_data(self.selector.extents): self._trigger_plot()
        except Exception as e: print(f"Drag error: {e}")

    def _on_select_finalize(self, eclick, erelease):
        try:
            if self._update_temp_roi_data(self.selector.extents):
                if self.app.live_plot_var.get(): self._trigger_plot()
        except Exception as e: print(f"Finalize error: {e}")

    def _on_poly_finalize(self, verts):
        mask = self._generate_mask("polygon", verts)
        if mask is None: return
        next_id = len(self.roi_list) + 1
        color = ROI_COLORS[(next_id - 1) % len(ROI_COLORS)]
        self.temp_roi = {'type': "polygon", 'params': verts, 'mask': mask, 'color': color, 'id_display': next_id}
        self._commit_temp_roi()

    def _create_high_contrast_roi(self, rtype, params, color):
        patches = []
        if rtype == "line":
            (x1, y1), (x2, y2) = params
            mid_pt = ((x1+x2)/2, (y1+y2)/2)
            
            # 1. 线条
            line_bg = mlines.Line2D([x1, x2], [y1, y2], color='white', linewidth=3, alpha=0.8)
            line_fg = mlines.Line2D([x1, x2], [y1, y2], color=color, linewidth=1.5, linestyle='-')
            
            # 2. [新增] 端点圆圈 (起点, 终点)
            # 半径设为 4 (数据坐标系下的半径，如果图像很大可能需要调整，或者改用 Marker)
            # 这里使用 Circle patch，它会随着 zoom 缩放
            c_start = Circle((x1, y1), radius=4, color=color, alpha=0.6)
            c_end   = Circle((x2, y2), radius=4, color=color, alpha=0.6)
            
            # 3. [新增] 中点圆圈 (用于平移)
            c_mid   = Circle(mid_pt, radius=3, color='white', alpha=0.9, linewidth=1, edgecolor='black')

            self.ax_ref.add_line(line_bg)
            self.ax_ref.add_line(line_fg)
            self.ax_ref.add_patch(c_start)
            self.ax_ref.add_patch(c_end)
            self.ax_ref.add_patch(c_mid)
            
            # 返回顺序很重要，拖动更新时会用到
            return [line_bg, line_fg, c_start, c_end, c_mid]

        fill_alpha = 0.6
        if rtype == "rect":
            x, y, w, h = params
            patches.append(Rectangle((x, y), w, h, linewidth=0, facecolor=color, alpha=fill_alpha))
            patches.append(Rectangle((x, y), w, h, linewidth=3, edgecolor='white', facecolor='none', linestyle='-'))
            patches.append(Rectangle((x, y), w, h, linewidth=2, edgecolor='black', facecolor='none', linestyle='--'))
        elif rtype == "circle":
            center, w, h = params
            patches.append(Ellipse(center, w, h, linewidth=0, facecolor=color, alpha=fill_alpha))
            patches.append(Ellipse(center, w, h, linewidth=3, edgecolor='white', facecolor='none', linestyle='-'))
            patches.append(Ellipse(center, w, h, linewidth=2, edgecolor='black', facecolor='none', linestyle='--'))
        elif rtype == "polygon":
            patches.append(Polygon(params, linewidth=0, facecolor=color, alpha=fill_alpha, closed=True))
            patches.append(Polygon(params, linewidth=3, edgecolor='white', facecolor='none', linestyle='-', closed=True))
            patches.append(Polygon(params, linewidth=2, edgecolor='black', facecolor='none', linestyle='--', closed=True))
            
        for p in patches:
            self.ax_ref.add_patch(p)
        return patches

    def _commit_temp_roi(self):
        if not self.temp_roi: return
        t = self.temp_roi
        if self.is_drawing_bg:
            self._process_background_roi(t)
            self.temp_roi = None
            self._stop_selector()
            self.is_drawing_bg = False 
            self.app.root.config(cursor="")
            return

        patch_group = self._create_high_contrast_roi(t['type'], t['params'], t['color'])
        if patch_group:
            self.roi_list.append({
                'type': t['type'], 
                'patch_group': patch_group, 
                'mask': t['mask'], 
                'color': t['color'], 
                'id': len(self.roi_list) + 1,
                'params': t['params']
            })
        
        self.temp_roi = None
        self.app.plot_mgr.canvas.draw_idle()
        if self.app.live_plot_var.get(): self._trigger_plot()

    def save_rois(self, filepath):
        if self.temp_roi: self._commit_temp_roi()
        self._stop_selector()
        if not self.roi_list:
            messagebox.showwarning("Save ROI", "No ROIs to save.")
            return
        
        data_to_save = []
        for roi in self.roi_list:
            item = {"type": roi['type'], "color": roi['color'], "id": roi['id']}
            if roi['type'] == "polygon": item["params"] = np.array(roi['params']).tolist() 
            else: item["params"] = roi['params']
            data_to_save.append(item)
            
        try:
            with open(filepath, 'w') as f: json.dump(data_to_save, f, indent=4)
            messagebox.showinfo("Success", f"Saved {len(data_to_save)} ROIs.")
        except Exception as e: messagebox.showerror("Error", f"Failed to save ROIs:\n{e}")

    def load_rois(self, filepath):
        if self.app.data1 is None:
            messagebox.showerror("Error", "Please load an image first.")
            return
        try:
            with open(filepath, 'r') as f: data_loaded = json.load(f)
            if not isinstance(data_loaded, list): raise ValueError("Invalid format.")

            self.clear_all() 
            for item in data_loaded:
                rtype = item["type"]
                params = item["params"]
                color = item.get("color", ROI_COLORS[0])
                if rtype == "polygon": params = np.array(params)
                else:
                    params = tuple(params)
                    if rtype == "circle": params = (tuple(params[0]), params[1], params[2])
                mask = None
                if rtype != "line":
                    mask = self._generate_mask(rtype, params)
                    if mask is None: continue
                patch_group = self._create_high_contrast_roi(rtype, params, color)
                if patch_group:
                    self.roi_list.append({
                        'type': rtype,
                        'patch_group': patch_group,
                        'mask': mask,
                        'color': color,
                        'id': len(self.roi_list) + 1,
                        'params': params
                    })
            self.app.plot_mgr.canvas.draw_idle()
            messagebox.showinfo("Success", f"Loaded {len(self.roi_list)} ROIs.")
        except Exception as e: messagebox.showerror("Error", f"Failed to load ROIs:\n{e}")

    def _generate_mask(self, shape_type, params):
        if self.app.data1 is None: return None
        h, w = self.app.data1.shape[1], self.app.data1.shape[2]
        try:
            if shape_type == "rect":
                xmin, ymin, width, height = params
                y, x = np.ogrid[:h, :w]
                x_start, x_end = int(max(0, xmin)), int(min(w, xmin + width))
                y_start, y_end = int(max(0, ymin)), int(min(h, ymin + height))
                if x_start >= x_end or y_start >= y_end: return None
                mask = np.zeros((h, w), dtype=bool)
                mask[y_start:y_end, x_start:x_end] = True
                return mask
            elif shape_type == "circle":
                center, width, height = params
                y, x = np.ogrid[:h, :w]
                return (((x - center[0]) / (width/2))**2 + ((y - center[1]) / (height/2))**2) <= 1
            elif shape_type == "polygon":
                verts = params
                y, x = np.mgrid[:h, :w]
                points = np.vstack((x.ravel(), y.ravel())).T
                mpl_path = MplPath(verts)
                return mpl_path.contains_points(points).reshape(h, w)
        except Exception: return None
        return None

    def remove_last(self):
        if self.current_shape_mode == "line" and self.line_start_pt:
            self.cancel_drawing()
            return
        if self.temp_roi:
            self.temp_roi = None
            self._stop_selector()
            return
        if self.roi_list:
            item = self.roi_list.pop()
            if 'patch_group' in item:
                for p in item['patch_group']:
                    try: p.remove()
                    except: pass
            self.app.plot_mgr.canvas.draw_idle()
        if self.app.live_plot_var.get(): self._trigger_plot()

    def clear_all(self):
        self.cancel_drawing() 
        self.temp_roi = None
        self.roi_list = []
        if self.ax_ref:
            for p in list(self.ax_ref.patches): p.remove()
            for l in list(self.ax_ref.lines): l.remove()
        if self.app.plot_mgr: self.app.plot_mgr.canvas.draw_idle()

    def _trigger_plot(self):
        if not self.is_calculating: self.plot_curve()

    def get_last_line_roi(self):
        for roi in reversed(self.roi_list):
            if roi['type'] == 'line':
                return roi
        return None

    def plot_curve(self, interval=1.0, unit='s', is_log=False, do_norm=False, int_thresh=0, ratio_thresh=0):
        if not self.roi_list and not self.temp_roi: return
        if self.is_calculating: return
        
        if not self.app.plot_mgr.plot_window_controller: return

        data_num, data_den, bg_num, bg_den = self.app.get_active_data()
        if data_num is None: return
        
        self.is_calculating = True
        
        data_aux_list = getattr(self.app, 'data_aux', [])
        bg_aux_list = getattr(self.app, 'cached_bg_aux', [])
        
        task_list = []
        for r in self.roi_list:
            task_list.append({'mask': r['mask'], 'color': r['color'], 'id': r['id']})
        if self.temp_roi and self.temp_roi.get('mask') is not None:
            task_list.append({'mask': self.temp_roi['mask'], 'color': self.temp_roi['color'], 'id': self.temp_roi['id_display']})

        if not task_list:
            self.is_calculating = False
            return

        threading.Thread(
            target=self._calc_multi_roi_thread, 
            args=(data_num, data_den, bg_num, bg_den, data_aux_list, bg_aux_list, interval, unit, is_log, do_norm, task_list, int_thresh, ratio_thresh)
        ).start()

    def _calc_multi_roi_thread(self, data_num, data_den, bg_num, bg_den, data_aux_list, bg_aux_list, interval, unit, is_log, do_norm, task_list, int_thresh, ratio_thresh):
        try:
            results = []
            
            def calc_dff(arr):
                valid_mask = arr > 1e-6
                if not np.any(valid_mask): return np.zeros_like(arr)
                valid_vals = arr[valid_mask]
                thresh_5 = np.percentile(valid_vals, 5)
                baseline_vals = valid_vals[valid_vals <= thresh_5]
                f0 = np.mean(baseline_vals) if len(baseline_vals) > 0 else np.mean(valid_vals)
                if f0 > 1e-6: return (arr - f0) / f0
                else: return np.zeros_like(arr)

            for item in task_list:
                mask = item['mask']
                if mask is None or np.sum(mask) == 0: continue

                y_idxs, x_idxs = np.where(mask)
                
                roi_num = data_num[:, y_idxs, x_idxs].astype(np.float32) - bg_num
                roi_num = np.clip(roi_num, 0, None)
                means_num = np.nanmean(roi_num, axis=1)
                means_num = np.nan_to_num(means_num, nan=0.0)

                if data_den is None:
                    means_ratio = means_num.copy()
                    means_den = np.zeros_like(means_num)
                else:
                    roi_den = data_den[:, y_idxs, x_idxs].astype(np.float32) - bg_den
                    roi_den = np.clip(roi_den, 0, None)
                    mask_valid = (roi_num > int_thresh) & (roi_den > int_thresh) & (roi_den > 0.001)
                    roi_ratio = np.full_like(roi_num, np.nan)
                    np.divide(roi_num, roi_den, out=roi_ratio, where=mask_valid)
                    if ratio_thresh > 0: roi_ratio[roi_ratio < ratio_thresh] = np.nan
                    means_ratio = np.nanmean(roi_ratio, axis=1)
                    means_ratio = np.nan_to_num(means_ratio, nan=0.0)
                    means_den = np.nanmean(roi_den, axis=1)
                    means_den = np.nan_to_num(means_den, nan=0.0)
                
                means_aux = []
                for i, d_aux in enumerate(data_aux_list):
                    bg_val = bg_aux_list[i] if i < len(bg_aux_list) else 0
                    roi_aux = d_aux[:, y_idxs, x_idxs].astype(np.float32) - bg_val
                    roi_aux = np.clip(roi_aux, 0, None)
                    m = np.nanmean(roi_aux, axis=1)
                    m = np.nan_to_num(m, nan=0.0)
                    if do_norm: m = calc_dff(m)
                    means_aux.append(m)

                if do_norm:
                    means_ratio = calc_dff(means_ratio)
                    means_num = calc_dff(means_num)
                    if data_den is not None: means_den = calc_dff(means_den)
                
                results.append({
                    'id': item['id'],
                    'color': item['color'],
                    'means': means_ratio,
                    'means_num': means_num,
                    'means_den': means_den,
                    'means_aux': means_aux
                })

            if not results: return

            mult = 1.0
            if unit == "m": mult = 1.0/60.0
            elif unit == "h": mult = 1.0/3600.0
            times = np.arange(len(results[0]['means'])) * interval * mult
            
            try: mode_var = self.app.ratio_mode_var.get()
            except: mode_var = "c1_c2"
            
            if data_den is None:
                labels = ("Intensity", "None")
                has_ratio = False
            else:
                labels = ("Ch1", "Ch2") if mode_var == "c1_c2" else ("Ch2", "Ch1")
                has_ratio = True
            
            aux_labels = []
            if data_aux_list:
                for i in range(len(data_aux_list)): aux_labels.append(f"Ch{i+3}")

            channel_info = {
                "labels": labels,
                "has_ratio": has_ratio,
                "aux_labels": aux_labels
            }

            self.app.root.after(0, lambda: self.app.plot_mgr.plot_window_controller.update_data(
                times, results, unit, is_log, do_norm, channel_info
            ))
            
        except Exception as e:
            print(f"Calc Error: {e}")
            import traceback
            traceback.print_exc()
        finally:
            self.is_calculating = False

    def _process_background_roi(self, roi_data):
        mask = roi_data['mask']
        if mask is None or self.app.data1 is None: return
        try:
            mean_img1 = np.mean(self.app.data1, axis=0) 
            bg_val1 = np.mean(mean_img1[mask])
            if self.app.data2 is not None:
                mean_img2 = np.mean(self.app.data2, axis=0)
                bg_val2 = np.mean(mean_img2[mask])
            else:
                bg_val2 = 0.0
            self.app.set_custom_background(float(bg_val1), float(bg_val2))
            if self.selector: self.selector.set_visible(False)
            self.app.plot_mgr.canvas.draw_idle()
        except Exception as e:
            tk.messagebox.showerror("Error", f"Failed to calc background: {e}")

    # =========================================================
    #  [新增] 支持工程文件保存/加载的接口
    # =========================================================

    def get_all_rois_data(self):
        """
        获取当前所有 ROI 的纯数据列表（用于保存到 Project 文件）。
        """
        if self.temp_roi: self._commit_temp_roi()
        self._stop_selector()
        
        data_list = []
        for roi in self.roi_list:
            item = {"type": roi['type'], "color": roi['color'], "id": roi['id']}
            if roi['type'] == "polygon": 
                # numpy array 转 list 以便 JSON 序列化
                item["params"] = np.array(roi['params']).tolist() 
            else: 
                item["params"] = roi['params']
            data_list.append(item)
        return data_list

    def restore_rois_from_data(self, data_list):
        """
        从数据列表恢复 ROI（用于从 Project 文件加载）。
        """
        if self.app.data1 is None:
            # 如果没有加载图像，无法计算 Mask，因此无法恢复 ROI
            print("Warning: Cannot restore ROIs. No image data loaded.")
            return

        # 清除现有 ROI
        self.clear_all() 

        if not isinstance(data_list, list):
            return

        for item in data_list:
            try:
                rtype = item["type"]
                params = item["params"]
                color = item.get("color", ROI_COLORS[0])

                # 参数类型转换 (JSON 加载回来通常是 list，需要转回 tuple 或 numpy)
                if rtype == "polygon": 
                    params = np.array(params)
                else:
                    params = tuple(params)
                    # 特殊处理圆形的参数结构: ((x,y), w, h)
                    if rtype == "circle": 
                        params = (tuple(params[0]), params[1], params[2])

                # 生成 Mask
                mask = None
                if rtype != "line":
                    mask = self._generate_mask(rtype, params)
                    if mask is None: continue

                # 创建视觉元素
                patch_group = self._create_high_contrast_roi(rtype, params, color)
                
                if patch_group:
                    self.roi_list.append({
                        'type': rtype,
                        'patch_group': patch_group,
                        'mask': mask,
                        'color': color,
                        'id': len(self.roi_list) + 1,
                        'params': params
                    })
            except Exception as e: 
                print(f"Error restoring ROI: {e}")
        
        self.app.plot_mgr.canvas.draw_idle()


# src/model.py
import numpy as np
import tifffile as tiff
import os
import warnings
from typing import List, Optional, Tuple, Any, Union

# 尝试相对导入 (作为包运行)，失败则尝试绝对导入 (直接运行脚本)
try:
    from .io_utils import read_and_split_multichannel, read_separate_files
    from .processing import calculate_background, process_frame_ratio
except ImportError:
    from io_utils import read_and_split_multichannel, read_separate_files
    from processing import calculate_background, process_frame_ratio

class AnalysisSession:
    """
    Model 层：管理 RIA 应用程序的核心数据状态和业务逻辑。
    
    职责:
    1. 存储图像数据 (data1, data2, aux)
    2. 管理计算参数 (阈值, 背景百分比等)
    3. 执行 I/O 操作 (读取文件, 解析元数据)
    4. 执行核心计算 (背景计算, 帧处理)
    """

    def __init__(self):
        # --- 核心图像数据 ---
        self.data1: Optional[np.ndarray] = None      # 分子 (Ch1 / Numerator)
        self.data2: Optional[np.ndarray] = None      # 分母 (Ch2 / Denominator)
        self.data_aux: List[np.ndarray] = []         # 辅助通道列表 (Ch3, Ch4...)
        
        # --- 原始数据备份 (用于撤销配准) ---
        self.data1_raw: Optional[np.ndarray] = None
        self.data2_raw: Optional[np.ndarray] = None 

        # --- 背景值缓存 ---
        self.cached_bg1: float = 0.0
        self.cached_bg2: float = 0.0
        self.cached_bg_aux: List[float] = [] 

        # --- 文件路径信息 ---
        self.c1_path: Optional[str] = None
        self.c2_path: Optional[str] = None
        self.dual_path: Optional[str] = None
        
        # --- 计算参数 ---
        self.bg_percent: float = 5.0  # 默认背景扣除百分比
        
        # --- 自定义背景 ROI 状态 ---
        self.custom_bg1: float = 0.0
        self.custom_bg2: float = 0.0
        # 注意：UI相关的 BooleanVar (如 use_custom_bg_var) 留在 GUI 里

        # --- 播放器与显示状态 ---
        self.fps: int = 10 
        self.is_playing: bool = False
        self.view_mode: str = "ratio" # ratio, ch1, ch2, aux_0...
        self.alignment_matrices = [] # [新增] 存储位移矩阵
        self.current_roles = None


    # [替换方法 1]
    def inspect_file_metadata(self, filepath: str) -> Tuple[bool, int, int, str]:
        """
        [修改] 检查文件的元数据。
        Returns:
            - is_multichannel (bool): 是否明确为多通道
            - channel_count (int): 通道数
            - z_slice_count (int): Z轴层数 (默认为1)
            - axes_str (str): 识别到的轴顺序 (如 'ZYX', 'TZCYX')
        """
        is_explicit_multichannel = False
        detected_channels = 0
        detected_z = 1
        detected_axes = "?"

        try:
            with tiff.TiffFile(filepath) as tif:
                
                # --- 获取 Axes 信息 ---
                if len(tif.series) > 0:
                    series = tif.series[0]
                    if hasattr(series, 'axes'):
                        detected_axes = series.axes  # 例如 'TZCYX' 或 'ZYX'

                # --- 检测逻辑 A: ImageJ Metadata ---
                if tif.imagej_metadata:
                    detected_channels = tif.imagej_metadata.get('channels', 1)
                    detected_z = tif.imagej_metadata.get('slices', 1) # ImageJ 'slices' 通常指 Z
                    
                    if detected_channels > 1:
                        is_explicit_multichannel = True

                # --- 检测逻辑 B: OME-XML 或 Axes 字符串 ---
                # 如果 ImageJ 元数据没读到，尝试从 Axes 字符串推断
                if hasattr(series, 'axes'):
                    # 尝试获取 C 维度
                    if 'C' in detected_axes:
                        c_index = detected_axes.find('C')
                        c_dim = series.shape[c_index]
                        if c_dim > 1:
                            is_explicit_multichannel = True
                            detected_channels = c_dim
                    
                    # 尝试获取 Z 维度
                    if 'Z' in detected_axes:
                        z_index = detected_axes.find('Z')
                        # 如果 ImageJ 没读到 Z，就信这个
                        if detected_z == 1: 
                            detected_z = series.shape[z_index]
                                
        except Exception as e:
            print(f"[Model Warning] Metadata inspection failed for {filepath}: {e}")
            
        return is_explicit_multichannel, detected_channels, detected_z, detected_axes
    
    

    # [替换方法 2]
    def load_channels_from_file(self, 
                                filepath: str, 
                                is_interleaved: bool, 
                                expected_channels: int,
                                z_proj_method: str = None,
                                user_axes: str = None) -> List[np.ndarray]: # [修改] 增加参数
        """
        从单个文件中读取并分离通道数据，支持 Z-Projection。
        """
        if not filepath or not os.path.exists(filepath):
            raise ValueError(f"File not found: {filepath}")
            
        # 调用底层工具
        channels = read_and_split_multichannel(
            filepath, 
            is_interleaved, 
            expected_channels,
            z_projection_method=z_proj_method,
            override_axes=user_axes # [修改] 传入覆写的 Axes
        )
        
        return channels


    def load_separate_channels(self, path1: str, path2: str) -> List[np.ndarray]:
        """
        从两个独立的文件分别读取 Ch1 和 Ch2。
        
        Args:
            path1 (str): Ch1 文件路径。
            path2 (str): Ch2 文件路径。
            
        Returns:
            List[np.ndarray]: [data_ch1, data_ch2]
        """
        if not path1 or not path2:
            raise ValueError("Both file paths must be provided.")
            
        # 调用底层工具
        d1, d2 = read_separate_files(path1, path2)
        
        return [d1, d2]

    def set_data(self, 
                 data_list: List[np.ndarray], 
                 roles: Optional[dict] = None) -> None:
        """
        将读取到的原始通道数据分配给 model 的属性 (data1, data2, data_aux)。
        
        Args:
            data_list (List[np.ndarray]): 原始通道数据列表。
            roles (dict, optional): 指定通道角色的字典。
                                    格式如 {'num': 0, 'den': 1}。
                                    如果不传，则根据列表长度自动分配。
        """
        count = len(data_list)

        # [新增] 记录当前的分配方案，以便保存到工程文件
        if roles:
            self.current_roles = roles
        else:
            # 如果没有指定 roles，则记录默认值 (0=Num, 1=Den)
            self.current_roles = {"num": 0, "den": 1} if count >= 2 else {"num": 0, "den": None}
        
        # 重置当前数据
        self.data1 = None
        self.data2 = None
        self.data_aux = []
        self.alignment_matrices = []
        
        # --- 情况 1: 单通道 ---
        if count == 1:
            self.data1 = data_list[0]
            self.data2 = None
            
        # --- 情况 2: 双通道 (最常见) ---
        elif count == 2:
            # 如果指定了 roles，即使只有2个通道也遵从 roles
            if roles:
                idx_num = roles.get("num", 0)
                idx_den = roles.get("den", 1)
                self.data1 = data_list[idx_num]
                self.data2 = data_list[idx_den]
            else:
                # 默认顺序
                self.data1 = data_list[0]
                self.data2 = data_list[1]
                
        # --- 情况 3: 多通道 (>2) ---
        else:
            if roles is None:
                # 默认分配：前两个为 Ratio 通道，其余为 Aux
                self.data1 = data_list[0]
                self.data2 = data_list[1]
                self.data_aux = data_list[2:]
            else:
                idx_num = roles["num"]
                idx_den = roles["den"]
                
                self.data1 = data_list[idx_num]
                self.data2 = data_list[idx_den]
                
                # 剩下的全部归入 Aux
                for i, d in enumerate(data_list):
                    if i != idx_num and i != idx_den:
                        self.data_aux.append(d)

        # 只要数据更新了，就清理掉旧的 Raw 备份 (用于对齐撤销的)
        self.data1_raw = None
        self.data2_raw = None
        
        
        self.recalc_background() # 自动重新计算背景
        self.alignment_matrices = [] #存储位移矩阵


    def align_data(self, progress_callback=None) -> None:
        """
        执行图像配准 (ECC 算法)。
        """
        try:
            from processing import align_stack_ecc
        except ImportError:
            try:
                from .processing import align_stack_ecc
            except ImportError:
                raise ImportError("OpenCV (cv2) is required for alignment.")

        if self.data1 is None:
            raise ValueError("No data to align.")

        # 1. 备份原始数据 (用于 Undo)
        if self.data1_raw is None:
            self.data1_raw = self.data1.copy()
            if self.data2 is not None:
                self.data2_raw = self.data2.copy()

        # 2. 准备配准目标
        target_data2 = self.data2 if self.data2 is not None else self.data1
        
        # 3. [修正] 调用算法 (只调用一次，接收 3 个返回值)
        # 删除之前多余的 d1_aligned, d2_aligned = align_stack_ecc(...) 调用
        d1_aligned, d2_aligned, matrices = align_stack_ecc(
            self.data1, 
            target_data2, 
            progress_callback=progress_callback
        )

        # 4. 更新 Model 状态
        self.data1 = d1_aligned
        if self.data2 is not None:
            self.data2 = d2_aligned

        self.alignment_matrices = matrices
            
        # 5. 配准后像素位置变了，必须重新计算背景值
        self.recalc_background()


    def undo_alignment(self) -> bool:
        """
        撤销配准操作，恢复原始数据。
        
        Returns:
            bool: 如果成功恢复返回 True，如果没有备份数据(说明没配准过)返回 False。
        """
        if self.data1_raw is not None:
            # 恢复数据
            self.data1 = self.data1_raw.copy()
            if self.data2_raw is not None:
                self.data2 = self.data2_raw.copy()
            
            # 清空备份 (表示回到了原始状态)
            self.data1_raw = None
            self.data2_raw = None
            self.alignment_matrices = []
            
            # 数据变了，重新计算背景
            self.recalc_background()
            return True
            
        return False


    def apply_existing_alignment(self, matrices_data):
        """
        [新增] 直接应用保存的矩阵 (Loading Project 时调用)
        """
        try:
            from processing import apply_alignment_matrices
        except: return

        if self.data1 is None: return

        # 1. 转换 JSON 列表回 Numpy 数组
        # JSON里存的是 list of lists, 我们需要 list of np.array
        matrices = [np.array(m, dtype=np.float32) for m in matrices_data]
        
        # 2. 备份原始数据
        if self.data1_raw is None:
            self.data1_raw = self.data1.copy()
            if self.data2 is not None: self.data2_raw = self.data2.copy()
            
        # 3. 快速应用
        self.data1 = apply_alignment_matrices(self.data1, matrices)
        if self.data2 is not None:
            self.data2 = apply_alignment_matrices(self.data2, matrices)
            
        # 4. 保存状态
        self.alignment_matrices = matrices
        self.recalc_background()

    def recalc_background(self) -> None:
        """
        根据当前的 bg_percent 参数，重新计算所有通道的背景值。
        """
        if self.data1 is None:
            return
        
        p = self.bg_percent
        
        # 计算 Data1 背景
        self.cached_bg1 = calculate_background(self.data1, p)
        
        # 计算 Data2 背景
        if self.data2 is not None:
            self.cached_bg2 = calculate_background(self.data2, p)
        else:
            self.cached_bg2 = 0.0
            
        # 计算 Aux 背景
        self.cached_bg_aux = []
        for aux in self.data_aux:
            val = calculate_background(aux, p)
            self.cached_bg_aux.append(val)

    def get_processed_frame(self, 
                            frame_idx: int, 
                            int_thresh: float = 0, 
                            ratio_thresh: float = 0,
                            smooth_size: int = 0,
                            log_scale: bool = False,
                            use_custom_bg: bool = False,
                            swap_channels: bool = False) -> Optional[np.ndarray]: # <--- [必须加上这一行]
        """
        [Pipeline] 获取处理后的一帧图像 (Intensity 或 Ratio)。
        
        Args:
            frame_idx (int): 帧索引。
            int_thresh (float): 强度阈值。
            ratio_thresh (float): 比率阈值。
            smooth_size (int): 平滑内核大小。
            log_scale (bool): 是否进行对数变换。
            use_custom_bg (bool): 是否使用自定义 ROI 背景 (而非全局百分位)。
            swap_channels (bool): 是否交换分子分母 (即 Ch2/Ch1)。
        
        Returns:
            Optional[np.ndarray]: 处理后的图像矩阵 (2D)。
        """
        if self.data1 is None:
            return None
        
        # 1. 确定背景值
        if use_custom_bg:
            bg1 = self.custom_bg1
            bg2 = self.custom_bg2
            # Aux 背景暂不支持自定义 ROI，仍使用全局
            bg_aux_list = self.cached_bg_aux 
        else:
            bg1 = self.cached_bg1
            bg2 = self.cached_bg2
            bg_aux_list = self.cached_bg_aux

        # [新增] 处理通道交换 (同时交换数据和背景)
        d_num = self.data1
        d_den = self.data2
        bg_num = bg1
        bg_den = bg2

        if swap_channels and self.data2 is not None:
            d_num = self.data2
            d_den = self.data1
            bg_num = bg2
            bg_den = bg1

        # 2. 根据 view_mode 决定返回什么
        
        # --- Case A: 查看原始通道 (Ch1) ---
        if self.view_mode == "ch1":
            # 返回：(数据 - 背景)，并 Clip 掉负值
            try:
                raw = self.data1[frame_idx].astype(np.float32) - bg1
                return np.clip(raw, 0, None)
            except IndexError:
                return None
            
        # --- Case B: 查看原始通道 (Ch2) ---
        elif self.view_mode == "ch2":
            if self.data2 is None: return None
            try:
                raw = self.data2[frame_idx].astype(np.float32) - bg2
                return np.clip(raw, 0, None)
            except IndexError:
                return None
            
        # --- Case C: 查看辅助通道 (Aux) ---
        elif self.view_mode.startswith("aux_"):
            try:
                idx = int(self.view_mode.split("_")[1])
                if idx < len(self.data_aux):
                    bg_val = bg_aux_list[idx] if idx < len(bg_aux_list) else 0
                    raw = self.data_aux[idx][frame_idx].astype(np.float32) - bg_val
                    return np.clip(raw, 0, None)
            except: 
                return None
            return None

        # --- Case D: Ratio / Intensity 模式 (默认) ---
        # 调用 processing 模块的核心算法
        return process_frame_ratio(
            d_num[frame_idx], 
            d_den[frame_idx] if d_den is not None else None,
            bg_num, bg_den,
            int_thresh, 
            ratio_thresh,
            smooth_size, 
            log_scale
        )
    
    # =========================================================================
    # Export / Save Logic
    # =========================================================================

    def export_processed_stack(self, 
                               filepath: str, 
                               params: dict, 
                               progress_callback=None) -> None:
        """
        将当前参数处理后的所有帧保存为多页 Tiff 文件。
        这个方法结合了 Processing (计算) 和 I/O (写入)，属于业务逻辑层。
        
        Args:
            filepath (str): 保存路径。
            params (dict): 处理参数 (int_thresh, ratio_thresh, smooth, log_scale 等)。
            progress_callback (callable, optional): 进度回调 (current, total)。
        """
        if self.data1 is None:
            raise ValueError("No data to save.")
            
        n_frames = self.data1.shape[0]
        
        # 使用 tifffile 的 Writer 来流式写入，节省内存
        # bigtiff=True 允许保存超过 4GB 的文件，适合长序列成像
        with tiff.TiffWriter(filepath, bigtiff=True) as tif:
            for i in range(n_frames):
                # 1. 核心：调用 Model 自身的处理管道获取图像
                frame_data = self.get_processed_frame(
                    frame_idx=i,
                    int_thresh=params.get("int_thresh", 0),
                    ratio_thresh=params.get("ratio_thresh", 0),
                    smooth_size=params.get("smooth", 0),
                    log_scale=params.get("log_scale", False),
                    use_custom_bg=params.get("use_custom_bg", False)
                )
                
                # 2. 写入文件
                if frame_data is not None:
                    tif.write(frame_data.astype(np.float32), contiguous=True)
                
                # 3. 报告进度
                if progress_callback and i % 5 == 0:
                    progress_callback(i, n_frames)
        
        # 最后报告一次完成
        if progress_callback:
            progress_callback(n_frames, n_frames)

    def export_raw_ratio_stack(self, 
                               filepath: str, 
                               int_thresh: float, 
                               ratio_thresh: float, 
                               progress_callback=None) -> None:
        """
        导出“纯净”的比率堆栈 (不含 Log 变换，不含平滑，仅做基础阈值过滤)。
        这通常用于需要将数据导入其他软件(如 ImageJ)进行进一步量化分析的场景。
        """
        if self.data1 is None:
            raise ValueError("No data to save.")

        n_frames = self.data1.shape[0]
        
        with tiff.TiffWriter(filepath, bigtiff=True) as tif:
            for i in range(n_frames):
                # 强制 smooth=0, log=False, use_custom_bg=False
                # 仅保留最基础的阈值过滤，保留数据的原始性
                frame_data = self.get_processed_frame(
                    frame_idx=i,
                    int_thresh=int_thresh,
                    ratio_thresh=ratio_thresh,
                    smooth_size=0, 
                    log_scale=False,
                    use_custom_bg=False
                )
                
                if frame_data is not None:
                    tif.write(frame_data.astype(np.float32), contiguous=True)

                if progress_callback and i % 5 == 0:
                    progress_callback(i, n_frames)
        
        if progress_callback:
            progress_callback(n_frames, n_frames)

    def export_current_frame(self, filepath: str, frame_idx: int, params: dict) -> None:
        """保存单帧图像"""
        img = self.get_processed_frame(
            frame_idx=frame_idx,
            int_thresh=params.get("int_thresh", 0),
            ratio_thresh=params.get("ratio_thresh", 0),
            smooth_size=params.get("smooth", 0),
            log_scale=params.get("log_scale", False),
            use_custom_bg=params.get("use_custom_bg", False)
        )
        if img is not None:
            tiff.imwrite(filepath, img)



# components.py
import tkinter as tk
from tkinter import ttk

class ToggledFrame(ttk.Frame):
    def __init__(self, parent, text="", *args, **options):
        # 默认使用 Card 样式，除非指定
        if "style" not in options:
            options["style"] = "Card.TFrame"
            
        ttk.Frame.__init__(self, parent, *args, **options)
        
        self.show = tk.IntVar()
        self.show.set(0)
        
        self.title_frame = ttk.Frame(self, style="Card.TFrame")
        self.title_frame.pack(fill="x", expand=1)
        
        self.toggle_btn = ttk.Checkbutton(
            self.title_frame, 
            width=2, 
            text='▶', 
            command=self.toggle, 
            variable=self.show, 
            style='Toolbutton'
        )
        self.toggle_btn.pack(side="left")
        
        self.lbl_title = ttk.Label(self.title_frame, text=text, style="Blue.TLabel")
        self.lbl_title.pack(side="left", padx=5)
        
        self.sub_frame = ttk.Frame(self, relief="flat", borderwidth=0, padding=5, style="Card.TFrame")

    def toggle(self):
        if self.show.get():
            self.sub_frame.pack(fill="x", expand=1, pady=(2,0))
            self.toggle_btn.configure(text='▼')
        else:
            self.sub_frame.forget()
            self.toggle_btn.configure(text='▶')


# src/io_utils.py
import tifffile as tiff
import numpy as np
import warnings
import os

def perform_z_projection(data, axis, method='max'):
    """
    对指定轴执行投影。
    method: 'max' (最大密度投影 MIP) 或 'ave' (平均密度投影 AIP)
    """
    print(f"Applying Z-Projection ({method}) on axis {axis}, original shape: {data.shape}")
    if method == 'max':
        return np.max(data, axis=axis)
    elif method == 'ave':
        # 保持原有数据类型 (如 uint16)
        return np.mean(data, axis=axis).astype(data.dtype)
    return data

def read_and_split_multichannel(file_path, is_interleaved, n_channels=2, z_projection_method=None, override_axes=None):
    """
    Universal reading function.
    override_axes: 如果用户手动指定了 Axes (如 'TYX')，则忽略文件自带的元数据。
    """
    try:
        with tiff.TiffFile(file_path) as tif:
            raw_data = tif.asarray()
            
            # [核心逻辑] 优先级：用户输入 > 文件自带 > 空字符串
            if override_axes:
                print(f"[IO] Using user override axes: {override_axes}")
                axes = override_axes
            else:
                axes = tif.series[0].axes if hasattr(tif.series[0], 'axes') else ""
                
    except Exception as e:
        raise ValueError(f"Could not read file: {e}")

    # =================================================================
    # Z-Stack 处理逻辑
    # =================================================================
    # 这里的逻辑非常巧妙：
    # 如果系统识别是 ZYX，用户手动改成了 TYX。
    # 此时 axes 变量变成了 "TYX"。
    # 下面的 'Z' in axes 判断就会失败。
    # 于是直接跳过投影，数据保留原样进入后续流程，完美解决问题！
    if z_projection_method and 'Z' in axes:
        z_index = axes.find('Z')
        if z_index < raw_data.ndim and raw_data.shape[z_index] > 1:
            raw_data = perform_z_projection(raw_data, axis=z_index, method=z_projection_method)

    channels = []


    # Logic 1: Interleaved Stack
    if is_interleaved:
        if raw_data.ndim != 3:
             # 如果选了 None (Treat as T)，原本 (Z,Y,X) 的 3D 数据会被保留
             # 这里 (Z,Y,X) 从 numpy 角度看就是 (Time, Y, X)，完全符合 3D Stack 要求
             # 所以这里的检查通过
             raise ValueError(f"Interleaved mode requires 3D stack (T, Y, X). Current shape: {raw_data.shape}")
        
        n_frames = raw_data.shape[0]
        if n_channels == 1:
            channels.append(raw_data)
        else:
            remainder = n_frames % n_channels
            if remainder != 0:
                raw_data = raw_data[:-remainder]
            for c in range(n_channels):
                channels.append(raw_data[c::n_channels])

    # Logic 2: Hyperstack (4D/3D)
    else:
        if raw_data.ndim == 4:
            # (T, C, Y, X)
            if raw_data.shape[1] >= 1 and raw_data.shape[1] <= 10: 
                for c in range(raw_data.shape[1]):
                    channels.append(raw_data[:, c, :, :])
            # (C, T, Y, X)
            elif raw_data.shape[0] >= 1 and raw_data.shape[0] <= 10 and raw_data.shape[1] > 10: 
                 for c in range(raw_data.shape[0]):
                     channels.append(raw_data[c, :, :, :])
            else:
                raise ValueError(f"Cannot identify channel dimension. Shape: {raw_data.shape}.")
                    
        elif raw_data.ndim == 3:
             # (T, Y, X)
             # [关键] 当选择了 None 投影，(90, 512, 512) 的数据会走到这里
             # 这时候它就被当作 90 帧的时间序列处理了。
             channels.append(raw_data)
        else:
            raise ValueError(f"Unsupported dimensions: {raw_data.shape}.")

    if not channels:
        raise ValueError("No channel data extracted.")
        
    min_len = min(len(c) for c in channels)
    channels = [c[:min_len] for c in channels]

    return channels




def read_separate_files(path1, path2):
    """
    读取两个独立的文件作为 Ch1 和 Ch2。
    """
    if not os.path.exists(path1) or not os.path.exists(path2):
        raise FileNotFoundError("One or both files not found.")

    with tiff.TiffFile(path1) as tif:
        d1 = tif.asarray()
    
    with tiff.TiffFile(path2) as tif:
        d2 = tif.asarray()

    # 简单的维度检查
    if d1.ndim == 2: d1 = d1[np.newaxis, ...] # 补齐 T 轴
    if d2.ndim == 2: d2 = d2[np.newaxis, ...]

    # 确保长度一致
    min_len = min(d1.shape[0], d2.shape[0])
    return d1[:min_len], d2[:min_len]


import sys
import os
import tkinter as tk
import ctypes
import argparse
import platform
import tifffile
import json       # [新增] 用于处理配置文件
import logging    # [新增] 用于调试日志

def print_welcome_art():
    """
    Prints the startup ASCII art banner for RIA (Liya).
    """
    banner = r'''
         _,addba,
         _,adP"'\  "Y,                       _____
       ,P"  d"Y,  \  8                  ,adP"""""""Yba,_
     ,d" /,d' `Yb, ,P'              ,adP"'           `""Yba,
     d'   d'    `"""         _,aadP"""""""Ya,             `"Ya,_
     8  | 8              _,adP"'                              `"Ya,
     8    I,           ,dP"           __              "baa,       "Yb,
     I,   Ya         ,db___           `"Yb,      a       `"         `"b,
     `Y, \ Y,      ,d8888888baa8a,_      `"      `"b,                 `"b,
      `Ya, `b,    d8888888888888888b,               "ba,                `8,
        "Ya,`b  ,d8888888888888888888,   d,           `"Ya,_             `Y,
          `Ybd8d8888888888888888888888b, `"Ya,            `""Yba,         `8,
             "Y8888888888888888888888888,   `Yb,               `"Ya        `b
              d8888888888888888888888888b,    `"'            ,    "b,       8,
              888888888888888888888888888b,                  b      "b      `b
              8888888888888888888888888888b    b,_           8       "       8
              I8888888888888888888888888888,    `"Yb,_       `b,             8
               Y888888888888888888888888888I        `Yb,       8,            8
                `Y8888888888888888888888888(          `8,       "b     a    ,P
                  "8888""Y88888888888888888I           `b,       `b    8    d'
                    "Y8b,  "Y888PPY8888888P'            `8,       P    8    8
                        `b   "'  __ `"Y88P'    b,        `Y       "    8    8
                       ""|      =""Y'   d'     `b,                     8    8
                        /         "' |  I       b             ,       ,P   ,P
                       (          _,"  d'       Y,           ,P       "    d'
                        |              I        `b,          d'            8
                        |              I          "         d,d'           8
                        |          ;   `b                  dP"          __,8_
                        |          |    `b                d"     _,,add8888888
                        ",       ,"      `b              d' _,ad88888888888888
                          \,__,a"          ",          _,add888888888888888888
                         _,aa888b           I       ,ad88888888888888888888888
                     _,ad88888888a___,,,gggd8,   ,ad88888888888888888888888888
                  ,ad888888888888b88PP""''  Y  ,dd8888888888888888888888888888
               ,ad8888888888888888'         `bd8888888888888888888888888888888
             ,d88888888888888888P'         ,d888888888888888888888888888888888
           ,d888888888888888888"         ,d88888888888888888888888888888888888
         ,d8888888888888888888P        ,d8888888888888888888888888888888888888
       ,d888888888888888888888b,     ,d888888888888888888888888888888888888888
      ,8888888888888888888888888888=888888888888888888888888888888888888888888
     d888888888888888888888888888=88888888888888888888888888888888888888888888
    d88888888888888888888888888=8888888888888888888888888888888888888888888888
   d8888888888888888888888888=888888888888888888888888888888888888888888888888
  d888888888888888888888888=88888888888888888888888888888888888888888888888888
 ,888888888888888888888888=888888888888888888888888888888888888888888888888888
 d8888888888888888888888=88888888888888888888888888888888888888888888888888888
,8888888888888888888888=888888888888888888888888888888888888888888888888888888
d888888888888888888888=88888888888888888888888888888888888888888 RIA (Liya) 88
888888888888888888888=888888888888888888888888888888888888888888 by Dr.Wang 88
888888888888888888888=88888888888888888888888888888888888888888888888888888888
    '''
    print(banner)
    print("                   RIA (Liya) says: Welcome!")
    print("----------------------------------------------------------------------\n")

# Add current directory to system path
current_dir = os.path.dirname(os.path.abspath(__file__))
sys.path.append(current_dir)

# Import core modules
try:
    from gui import RatioAnalyzerApp
    from _version import __version__
except ImportError:
    try:
        from src.gui import RatioAnalyzerApp
        from src._version import __version__
    except ImportError as e:
        print(f"Error importing core modules: {e}")
        raise

# ==========================================
# [新增] 功能处理函数
# ==========================================

def handle_init_config():
    """
    生成默认配置文件 ria_config.json。
    这允许用户在不修改代码的情况下自定义启动参数。
    """
    config_path = os.path.join(os.getcwd(), "ria_config.json")
    
    if os.path.exists(config_path):
        print(f"ℹ️  Config file already exists: {config_path}")
        print("   (Delete it first if you want to regenerate defaults)")
        sys.exit(0)

    # 定义默认配置结构
    default_config = {
        "version": __version__,
        "processing": {
            "default_int_threshold": 0,
            "default_ratio_threshold": 0.0,
            "default_smooth_size": 0,
            "bg_subtraction_percent": 5.0
        },
        "display": {
            "default_colormap": "coolwarm",
            "default_bg_color": "Trans",
            "default_fps": 10
        },
        "paths": {
            "auto_load_last_project": False,
            "default_export_dir": "./output"
        }
    }

    try:
        with open(config_path, 'w', encoding='utf-8') as f:
            json.dump(default_config, f, indent=4)
        print(f"✅ Generated default config file: {config_path}")
        print("   You can now edit this file to customize RIA startup behavior.")
    except Exception as e:
        print(f"❌ Failed to create config file: {e}")
    
    sys.exit(0)

def handle_sysinfo():
    """打印系统环境信息，用于调试"""
    print("\n=== RIA System Info ===")
    print(f"RIA Version:   {__version__}")
    print(f"Python:        {sys.version.split()[0]} ({platform.architecture()[0]})")
    print(f"OS:            {platform.system()} {platform.release()}")
    
    try:
        import numpy
        print(f"NumPy:         {numpy.__version__}")
    except ImportError:
        print("NumPy:         Not Installed")
        
    try:
        import cv2
        print(f"OpenCV:        {cv2.__version__}")
    except ImportError:
        print("OpenCV:        Not Installed (Alignment disabled)")

    try:
        import matplotlib
        print(f"Matplotlib:    {matplotlib.__version__}")
    except ImportError:
        print("Matplotlib:    Not Installed")
        
    print("=======================\n")
    sys.exit(0)

def handle_file_info(filepath):
    """快速读取 Tiff 文件头信息而不加载整个文件"""
    if not os.path.exists(filepath):
        print(f"Error: File not found: {filepath}")
        sys.exit(1)

    print(f"\nScanning: {filepath} ...")
    try:
        with tifffile.TiffFile(filepath) as tif:
            series = tif.series[0]
            shape = series.shape
            dtype = series.dtype
            axes = series.axes
            
            ij_meta = tif.imagej_metadata
            channels = 1
            frames = 1
            slices = 1
            if ij_meta:
                channels = ij_meta.get('channels', 1)
                slices = ij_meta.get('slices', 1)
                frames = ij_meta.get('frames', 1)
            
            print(f"--- File Metadata ---")
            print(f"Dimensions:         {shape}")
            print(f"Data Type:          {dtype}")
            print(f"Axes:               {axes}")
            if ij_meta:
                print(f"ImageJ Metadata:    T={frames}, Z={slices}, C={channels}")
            
            size_mb = os.path.getsize(filepath) / (1024 * 1024)
            print(f"File Size:          {size_mb:.2f} MB")
            print("---------------------\n")
            
    except Exception as e:
        print(f"Error reading file metadata: {e}")
    
    sys.exit(0)

# ==========================================
# Main
# ==========================================

def main():
    parser = argparse.ArgumentParser(
        description="RIA (Ratio Imaging Analyzer): A tool for ratiometric imaging analysis.",
        prog="ria"
    )

    # 1. 版本
    parser.add_argument('-v', '--version', action='version', version=f'RIA {__version__}', help="Show version.")

    # 2. 系统信息
    parser.add_argument('-s', '--sysinfo', action='store_true', help="Show system dependencies and environment info.")

    # 3. 文件信息
    parser.add_argument('-i', '--info', metavar='FILE', help="Inspect a Tiff file's metadata without opening GUI.")

    # 4. [新增] 调试模式
    parser.add_argument('-d', '--debug', action='store_true', help="Enable debug mode with verbose logging.")

    # 5. [新增] 初始化配置
    parser.add_argument('-ini', '--init', action='store_true', help="Generate default configuration file (ria_config.json).")

    # 6. 启动文件
    parser.add_argument('filename', nargs='?', help="Path to image/project to open.")

    args = parser.parse_args()

    # --- 1. 处理日志配置 (最先执行) ---
    if args.debug:
        logging.basicConfig(
            level=logging.DEBUG,
            format='%(asctime)s - %(levelname)s - %(message)s',
            datefmt='%H:%M:%S'
        )
        logging.debug("🔧 Debug Mode Enabled")
        logging.debug(f"Arguments parsed: {args}")
    else:
        # 默认只显示警告及以上
        logging.basicConfig(level=logging.WARNING, format='%(levelname)s: %(message)s')

    # --- 2. 处理功能指令 ---
    if args.sysinfo:
        handle_sysinfo()

    if args.init:
        handle_init_config()

    if args.info:
        handle_file_info(args.info)

    # --- 3. 启动 GUI ---
    print_welcome_art()
    print(f"Starting Ratio Imaging Analyzer {__version__}...")

    if os.name == 'nt':
        try:
            myappid = f'epivitae.ratioimaginganalyzer.ria.{__version__}.v3'
            ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
        except Exception as e:
            logging.warning(f"Could not set AppUserModelID: {e}")

    root = tk.Tk()

    startup_file = None
    if args.filename:
        candidate = os.path.abspath(args.filename)
        if os.path.exists(candidate):
            startup_file = candidate
            print(f"Startup file detected: {startup_file}")
            logging.debug(f"Startup file found: {startup_file}")
        else:
            print(f"Warning: File provided but not found: {candidate}")
            logging.warning(f"File not found: {candidate}")

    # 启动应用
    try:
        app = RatioAnalyzerApp(root, startup_file=startup_file)
        root.mainloop()
    except Exception as e:
        logging.critical(f"Unhandled exception in Main Loop: {e}", exc_info=True)
        raise

if __name__ == "__main__":
    main()


# src/processing.py
import numpy as np

# [核心依赖] 必须安装 OpenCV
try:
    import cv2
except ImportError:
    cv2 = None

def calculate_background(stack_data, percentile):
    """
    计算背景值。
    """
    if stack_data is None:
        return 0.0
    return np.nanpercentile(stack_data, percentile)

def smooth_nan_safe(arr, size):
    """
    平滑处理 (Smoothing)。
    [优化] 仅依赖 OpenCV，移除 SciPy 依赖以瘦身。
    """
    if size <= 1: return arr
    
    # 如果没有 OpenCV，直接报错提示用户安装，不再依赖 scipy
    if cv2 is None:
        raise ImportError("Missing Dependency: Please install 'opencv-python' to use Smoothing features.")

    # 确保是 float32
    arr = arr.astype(np.float32)
    k = int(size)
    
    # 1. 生成有效像素掩膜 (1.0 = Valid, 0.0 = NaN)
    valid_mask = (~np.isnan(arr)).astype(np.float32)
    
    # 2. 填充 NaN 为 0
    arr_filled = np.nan_to_num(arr, nan=0.0)
    
    # 3. 归一化卷积 (Normalized Convolution)
    blurred_img = cv2.blur(arr_filled, (k, k))
    blurred_mask = cv2.blur(valid_mask, (k, k))
    
    # 4. 计算结果
    with np.errstate(divide='ignore', invalid='ignore'):
        result = blurred_img / blurred_mask
        
    # 5. 清理无效区域
    result[blurred_mask < 1e-6] = np.nan
    
    return result

def process_frame_ratio(d1_frame, d2_frame, bg1, bg2, int_thresh, ratio_thresh, smooth_size, log_scale=False):
    """
    核心比率计算函数。
    [修改] 增加了单通道模式支持 (d2_frame is None)。
    """
    # 1. 类型转换 float32 防止下溢
    img1 = d1_frame.astype(np.float32) - bg1
    img1 = np.clip(img1, 0, None)

    # [修改] 单通道模式检测
    if d2_frame is None:
        # 单通道模式：仅做强度阈值处理
        if int_thresh > 0:
            img1[img1 < int_thresh] = np.nan # 或者 0，视需求而定，这里用 NaN 保持背景干净
        
        # 单通道通常不需要 Log Scale，但如果你想保留也可以
        # if log_scale: img1 = np.log1p(img1)
        return img1

    # --- 以下为原本的双通道比率逻辑 ---
    img2 = d2_frame.astype(np.float32) - bg2
    img2 = np.clip(img2, 0, None)

    # 3. 计算 Ratio
    with np.errstate(divide='ignore', invalid='ignore'):
        ratio = np.full_like(img1, np.nan)
        np.divide(img1, img2, out=ratio, where=img2 > 0.001)
    
    # 4. 阈值处理
    if int_thresh > 0:
        mask_low = (img1 < int_thresh) | (img2 < int_thresh)
        ratio[mask_low] = np.nan

    if ratio_thresh > 0:
        ratio[ratio < ratio_thresh] = np.nan
        
    # 5. 平滑处理
    if smooth_size > 1:
        ratio = smooth_nan_safe(ratio, int(smooth_size))

    # 6. Log 显示
    if log_scale:
        ratio = np.log1p(ratio)
        
    return ratio


def align_stack_ecc(data1, data2, progress_callback=None):
    """
    [修改] 返回值增加了 matrices 列表
    Returns: (aligned_1, aligned_2, matrices_list)
    """
    if cv2 is None: raise ImportError("OpenCV required.")

    frames, h, w = data1.shape
    
    # 简单的参考帧选择逻辑
    mean1 = np.nanmean(data1)
    mean2 = np.nanmean(data2)
    is_c1_ref = (mean1 >= mean2)
    stack_ref = data1 if is_c1_ref else data2
    stack_move = data2 if is_c1_ref else data1

    aligned_ref = np.zeros((frames, h, w), dtype=np.float32)
    aligned_move = np.zeros((frames, h, w), dtype=np.float32)
    
    template = np.nan_to_num(stack_ref[0].astype(np.float32), nan=0.0)
    aligned_ref[0] = template
    aligned_move[0] = stack_move[0].astype(np.float32)

    # 存储矩阵
    # 格式：List of 2x3 numpy arrays
    matrices = []
    
    # 第0帧是单位矩阵 (无位移)
    eye_matrix = np.eye(2, 3, dtype=np.float32)
    matrices.append(eye_matrix)

    criteria = (cv2.TERM_CRITERIA_EPS | cv2.TERM_CRITERIA_COUNT, 50, 1e-5)
    motion_mode = cv2.MOTION_TRANSLATION # 或者 MOTION_EUCLIDEAN
    
    # 当前的变换矩阵 (累积)
    warp_matrix = np.eye(2, 3, dtype=np.float32)

    for i in range(1, frames):
        current_img = np.nan_to_num(stack_ref[i].astype(np.float32), nan=0.0)

        try:
            # 计算矩阵
            (_, warp_matrix) = cv2.findTransformECC(
                template, current_img, warp_matrix, motion_mode, criteria, None, 5
            )
            
            # 应用矩阵
            aligned_ref[i] = cv2.warpAffine(
                stack_ref[i].astype(np.float32), warp_matrix, (w, h), 
                flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP,
                borderMode=cv2.BORDER_CONSTANT, borderValue=np.nan 
            )
            aligned_move[i] = cv2.warpAffine(
                stack_move[i].astype(np.float32), warp_matrix, (w, h), 
                flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP,
                borderMode=cv2.BORDER_CONSTANT, borderValue=np.nan
            )
            
            # 保存该帧的矩阵 (深拷贝)
            matrices.append(warp_matrix.copy())

        except cv2.error:
            # 配准失败，使用上一帧的矩阵或单位矩阵
            # 这里简单处理：不做变换
            aligned_ref[i] = stack_ref[i].astype(np.float32)
            aligned_move[i] = stack_move[i].astype(np.float32)
            matrices.append(np.eye(2, 3, dtype=np.float32)) # 记录无位移

        if progress_callback: progress_callback(i, frames)

    # 返回结果：根据谁是参考帧，决定返回顺序
    # 重要：我们也需要保存“谁是参考帧”的信息，但为了简化，我们假设矩阵是用于 warp_inverse 的
    if is_c1_ref: 
        return aligned_ref, aligned_move, matrices
    else: 
        return aligned_move, aligned_ref, matrices





def apply_alignment_matrices(data, matrices):
    """
    [新增] 快速应用已知的矩阵列表
    """
    if cv2 is None: raise ImportError("OpenCV required.")
    if data is None: return None
    
    frames, h, w = data.shape
    aligned = np.zeros_like(data, dtype=np.float32)
    
    # 确保矩阵数量匹配
    count = min(frames, len(matrices))
    
    for i in range(count):
        mat = matrices[i]
        src = data[i].astype(np.float32)
        
        # 检查是否是单位矩阵 (无需变换)
        if np.allclose(mat, np.eye(2, 3, dtype=np.float32)):
            aligned[i] = src
        else:
            aligned[i] = cv2.warpAffine(
                src, mat, (w, h),
                flags=cv2.INTER_LINEAR + cv2.WARP_INVERSE_MAP,
                borderMode=cv2.BORDER_CONSTANT, borderValue=np.nan
            )
            
    return aligned




def extract_kymograph(stack, p1, p2):
    """
    从图像堆栈中提取沿直线的 Kymograph 数据。
    stack: (Frames, Height, Width)
    p1: (x1, y1) 起点
    p2: (x2, y2) 终点
    返回: (Frames, Distance) 的 2D 矩阵
    """
    import numpy as np
    
    # 1. 计算两点间距离 (作为 Kymograph 的宽度)
    x1, y1 = p1
    x2, y2 = p2
    length = int(np.hypot(x2 - x1, y2 - y1))
    
    if length == 0: return None

    # 2. 生成采样坐标 (使用线性插值)
    x_coords = np.linspace(x1, x2, length)
    y_coords = np.linspace(y1, y2, length)
    
    # 3. 转换为整数索引并限制在图像范围内
    h, w = stack.shape[1], stack.shape[2]
    x_idxs = np.clip(x_coords.astype(int), 0, w - 1)
    y_idxs = np.clip(y_coords.astype(int), 0, h - 1)
    
    # 4. 利用 NumPy 高级索引一次性提取所有帧的数据
    # stack 形状为 (T, Y, X)，我们提取所有 T，特定的 Y 和 X
    kymo_matrix = stack[:, y_idxs, x_idxs]
    
    return kymo_matrix





# src/constants.py

LANG_MAP = {
    "window_title": {"cn": "比率成像分析器 ({})", "en": "Ratio Imaging Analyzer ({})"},
    "header_title": {"cn": "Ratio Imaging Analyzer (RIA)", "en": "Ratio Imaging Analyzer (RIA)"},

    "grp_file": {"cn": "1. 文件加载", "en": "1. File Loading"},

    "grp_pre": {"cn": "2. 图像配准 (可选)", "en": "2. Image Registration (Optional)"},
    "btn_align": {"cn": "✨ 运行运动校正", "en": "✨ Run Motion Correction"},

    "btn_align_done": {"cn": "✔ 配准完成", "en": "✔ Reg. Complete"},
    "btn_undo_align": {"cn": "↩ 撤销", "en": "↩ Undo"},

    "btn_undo_done": {"cn": "✔ 已撤销", "en": "✔ Undone"},
    
    "lbl_align_info": {"cn": "基于 Ch1 校正位移 (需 OpenCV)", "en": "Aligns stack based on Ch1."},
    "msg_aligning": {"cn": "正在进行 ECC 配准...", "en": "Running ECC Alignment..."},

    
    "grp_calc": {"cn": "3. 参数校准", "en": "3. Calibration"},
    

    "grp_view": {"cn": "4. 显示设置", "en": "4. Display Settings"},


    "tab_sep": {"cn": " 分别导入 (两文件) ", "en": " Separate Files "},
    "tab_dual": {"cn": " 单文件多通道 ", "en": " Single File "},
    
    "btn_c1": {"cn": "📂 通道 1", "en": "📂 Ch1"},
    "btn_c2": {"cn": "📂 通道 2", "en": "📂 Ch2"},
    "btn_dual": {"cn": "📂 选择多通道文件", "en": "📂 Select File"},
    "chk_interleaved": {"cn": "交错堆栈", "en": "Mixed Stacks"},
    
    "btn_load": {"cn": "🚀 加载并分析", "en": "🚀 Load & Analyze"},
    "lbl_no_file": {"cn": "...", "en": "..."},
    
    "lbl_int_thr": {"cn": "强度阈值", "en": "Int. Min"},
    "lbl_ratio_thr": {"cn": "比率阈值", "en": "Ratio Min"},
    "lbl_smooth": {"cn": "平滑 (Smooth)", "en": "Smooth"},
    "lbl_bg": {"cn": "背景扣除 %", "en": "BG %"},
    "chk_log": {"cn": "📈 Log (对数显示)", "en": "📈 Log Scale"},

    "lbl_cmap": {"cn": "伪彩:", "en": "Colormap:"},
    "lbl_bg_col": {"cn": "背景色:", "en": "BG Color:"},
    "chk_lock": {"cn": "🔒 锁定范围", "en": "🔒 Lock Range"},
    "btn_apply": {"cn": "应用", "en": "Apply"},
    "lbl_roi_tools": {"cn": "🛠️ ROI & 测量", "en": "🛠️ ROI & Measurement"},
    "lbl_export": {"cn": "💾 数据导出", "en": "💾 Data Export"},
    "lbl_settings": {"cn": "⚙️ 其他设置", "en": "⚙️ Settings"},
    "btn_draw": {"cn": "✏️ 新建 (Ctrl+T)", "en": "✏️ New (Ctrl+T)"},
    "btn_clear": {"cn": "🗑️ 清除", "en": "🗑️ Clear"},
    "btn_plot": {"cn": "📈 生成曲线 (Ctrl+P)", "en": "📈 Plot Curve (Ctrl+P)"},
    "btn_save_stack": {"cn": "💾 保存序列 (Stack)", "en": "💾 Save Stack"},
    "btn_save_raw": {"cn": "💽 保存原始比值", "en": "💽 Save Raw Ratio"}, 
    "btn_save_frame": {"cn": "📷 保存当前帧", "en": "📷 Save Frame"}, 
    "chk_live": {"cn": "🔴 实时监测 (Ctrl+L)", "en": "🔴 Live Monitor (Ctrl+L)"},
    "lbl_interval": {"cn": "Imaging Interval (s):", "en": "Imaging Interval (s):"}, 
    "lbl_unit": {"cn": "Plotting Unit:", "en": "Plotting Unit:"},
    "lbl_speed": {"cn": "倍速:", "en": "Speed:"},
    "btn_copy_all": {"cn": "📋 复制全部数据", "en": "📋 Copy All"},
    "btn_copy_y": {"cn": "🔢 仅复制 Ratio", "en": "🔢 Copy Ratio"},
    "btn_check_update": {"cn": "🔄 检查更新", "en": "🔄 Check Update"},
    "btn_contact": {"cn": "📧 联系作者", "en": "📧 Contact Author"},
    "msg_uptodate": {"cn": "当前已是最新版本！", "en": "You are up to date!"},
    "msg_new_ver": {"cn": "发现新版本: {}\n是否前往下载？", "en": "New version found: {}\nGo to download page?"},
    "title_update": {"cn": "版本更新", "en": "Update Check"},
    "err_check": {"cn": "检查更新失败: ", "en": "Check failed: "},
    "lbl_shape": {"cn": "ROI:", "en": "ROI:"},
    "lbl_ratio_mode": {"cn": "比率模式:", "en": "Ratio Mode:"},
    "mode_c1_c2": {"cn": "Ch1 / Ch2 (默认)", "en": "Ch1 / Ch2 (Default)"},
    "mode_c2_c1": {"cn": "Ch2 / Ch1 (反转)", "en": "Ch2 / Ch1 (Inverted)"},
}